--[[
	Basic RuneWatch Module Object
	Author: voorije
	Last Edited: 2009-01-15T13:13:54Z
]]

assert(RuneWatch, "Unable to find RuneWatch Object.");
local L = LibStub("AceLocale-3.0"):GetLocale("RuneWatch");

local Module = 
{ 
	ID = "",
	Name = "",
	Config = nil
}

function Module:Super(t)
	local sup = getmetatable(self)["__index"]
	return sup[t](sup == Module and self or sup)
end

function Module:IsDeathKnight()
	return RuneWatch:IsDeathKnight();
end

function Module:Register(sub, mName, mCfg, mDefaults)
	local mID = sub.name or sub.moduleName;
	self.ModID = mID;
	self.ModName = mName;
	self.ModConfig = mConfig;
	
	RuneWatch:RegisterModule(mID, self);
	
	if (mCfg) then
		RuneWatch:InitModuleOptions(mID, mName, mCfg);
	end
	
	if (mDefaults) then
		sub.dbase = RuneWatch.dbase:RegisterNamespace(mID, mDefaults);
		sub.db = sub.dbase.profile
	end
end

function Module:RegisterDrag(parent)
	local mID = self.ModID;
	
	local drag = CreateFrame("frame", "$parent_Drag", parent);
	drag:SetFrameStrata("HIGH");
	drag:SetFrameLevel(100);
	drag:EnableMouse(true);
	drag:Show();
	
	local tex = drag:CreateTexture("$parent_T_Green", "OVERLAY");
	tex:SetTexture(0, 0.6, 0, 0.5);
	tex:Show();
	tex:SetAllPoints(drag);
	
	local txt = drag:CreateFontString("$parent_F_Text", "HIGHLIGHT", "GameFontHighlight");
	txt:SetText("Drag");
	txt:SetAllPoints(drag);
	txt:SetJustifyH("CENTER");
	txt:SetJustifyV("MIDDLE");
	txt:Show();
	
	drag.Background = tex;
	drag.Text = txt;
	
	drag.id = mID;
	drag.parent = parent;
	
	drag:SetScript("OnMouseDown", function(self, ...)
		--RuneWatch:Print("Dragable: Moving");
		self.parent:StartMoving();
	end);
	
	drag:SetScript("OnMouseUp", function(self, ...)
		--RuneWatch:Print("Dragable: Stopped");
		self.parent:StopMovingOrSizing();
		
		for i = 1,self.parent:GetNumPoints() do
			local pt, _, _, x, y = self.parent:GetPoint(i);
			if (pt == "CENTER") then
				RuneWatch:SaveDragPosition(mID, x, y);
				break;
			end
		end
		
	end);
	
	-- Dragables
	if (not RuneWatch.Dragables[mID]) then
		RuneWatch.Dragables[mID] = {
			id = mID,
			dragFrame = drag,
			baseFrame = parent
		}
	end
	
	-- Reposition
	local xx, yy = RuneWatch:LoadDragPosition(mID);
	if (xx) then
		parent:ClearAllPoints();
		parent:SetPoint("CENTER", UIParent, "CENTER", xx, yy);
	end
	
	return drag;
end

function Module:GetDB()
	
end

RuneWatch.ModuleBase = Module;
RuneWatch.Modules = {}
RuneWatch.Dragables = {}

--[======= API Functions =======]--
function RuneWatch:SaveDefaultDragPosition(id, xp, yp)
	if (not RuneWatch.db.drags[id]) then
		self:SaveDragPosition(id, xp, yp);
	end
end

function RuneWatch:SaveDragPosition(id, xp, yp)
	local d = { x = xp, y = yp }
	
	RuneWatch.db.drags[id] = d;
end

function RuneWatch:LoadDragPosition(id)
	if (RuneWatch.db.drags[id]) then
		local d = RuneWatch.db.drags[id];
		return d.x, d.y;
	end
	
	return nil;
end

function RuneWatch:RegisterModule(module, object)
	local mods = RuneWatch.Modules;
	
	mods[module] = object;
	
	-- Create a module DB if it dosen't exist yet.
	if (RuneWatch.db.modules[module] == nil) then
		RuneWatch.db.modules[module] = true;
	end
end

function RuneWatch:GetModuleObject(module)
	local mods = RuneWatch.Modules;
	
	if (mods[module]) then
		return mods[module];
	end
	
	return nil;
end


