--[[
  Title: RuneWatch v2.0.0 Beta 5: Disease Tracker
  Author: voorije
  Revised: 2009-01-22T17:32:41Z
--]]

local AceDB = LibStub("AceDB-3.0")
--local AceAddon = LibStub("AceAddon-3.0");

-- Find RuneWatch Object
assert(RuneWatch, "Unable to find RuneWatch Object.");

-- Find Ace3
--assert(AceAddon, "Unable to load AceAddon-3.0");
assert(AceDB, "Unable to load AceAddon-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("RuneWatch_DiseaseTracker");

local DISEASE_BP = 1;
local DISEASE_FF = 2;
local DISEASE_CF = 3;
local DISEASE_EP = 4;
local DISEASE_UB = 5;

local HasEP = false;

local Disease = RuneWatch:NewModule("DiseaseTracker", RuneWatch.ModuleBase, "AceConsole-3.0", "AceEvent-3.0")
Disease.Frame = CreateFrame("frame", "RuneWatch_DT", RuneWatchFrame);
Disease.Frame:SetPoint("CENTER", UIParent, "CENTER", 0, 80);
Disease.Frame:SetWidth(10);
Disease.Frame:SetHeight(10);
Disease.Frame:Show();
Disease.Frame:SetMovable(true);

Disease.TargetGUID = "0x0";

RW_TrackerFont = CreateFont("RW_TrackerFont");
RW_TrackerFont:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE");
RW_TrackerFont:SetJustifyV("MIDDLE");
RW_TrackerFont:SetJustifyH("CENTER");

local WidgetArt = {
	Disease = "Interface\\ICONS\\Spell_Nature_NullifyDisease",
	BP = "Interface\\ICONS\\Spell_DeathKnight_BloodPlague",
	FF = "Interface\\ICONS\\Spell_DeathKnight_FrostFever",
	CF = "Interface\\ICONS\\Spell_DeathKnight_CryptFever",
	EP = "Interface\\ICONS\\Spell_DeathKnight_EbonPlague",
	UB = "Interface\\ICONS\\Spell_DeathKnight_UnholyBlight",
}

local WIcons = {
	BP = "Interface\\AddOns\\RuneWatch_DiseaseTracker\\Artwork\\Icon-BloodPlague",
	FF = "Interface\\AddOns\\RuneWatch_DiseaseTracker\\Artwork\\Icon-FrostFever",
	CF = "Interface\\AddOns\\RuneWatch_DiseaseTracker\\Artwork\\Icon-CryptFever",
	EP = "Interface\\AddOns\\RuneWatch_DiseaseTracker\\Artwork\\Icon-EbonPlague",
	UB = "Interface\\AddOns\\RuneWatch_DiseaseTracker\\Artwork\\Icon-Unblight",
	Ring = "Interface\\AddOns\\RuneWatch_DiseaseTracker\\Artwork\\Icon-Ring"
}

local SpellIDs = {
	BP = 55078,
	FF = 55095,
	CF = 50509,
	EP = 51734,
	UB = 55855
}

local SpellNames = {
	BP = "",
	FF = "",
	CF = "",
	EP = "",
	UB = ""
}

local WarningColors = {
	[1] = { 0.7, 0.5, 0 },
	[2] = { 0.5, 0.5, 1 },
}

local DiseaseConfig = {
	type = "group",
	name = L["Disease Tracker"],
	desc = L["Disease Tracker"],
	args = {
		warnings = {
			type = "toggle",
			name = L["Enable Warnings"],
			desc = L["Enable Scrolling Combat Warnings for the disease tracker"],
			get = function(info) return Disease.db.warningsEnabled; end,
			set = function(info, v) Disease.db.warningsEnabled = v; end,
		}
	}
}

function Disease:OnUpdate(self, elapsed)
	Disease.UpdateElapsed = Disease.UpdateElapsed + elapsed;
	
	if (Disease.UpdateElapsed > Disease.UpdateInterval) then
		Disease.UpdateElapsed = 0;
		
		if (Disease.db.warningsEnabled) then
			local _state = Disease.State;
			
			-- Check Blood Plague
			local _s = Disease.State.BP;
			local _r = _s.Time - GetTime(); -- Time Remaining
			if (_s.Active) then
				if (_r > 5 and _state.BPWarned) then
					_state.BPWarned = false;
				elseif (_r <= 5 and not _state.BPWarned) then
					_state.BPWarned = true;
					RuneWatch.Core:ScrollEvent("<"..L["Blood Plague Fading"]..">", unpack(WarningColors[DISEASE_BP]));
				end
			end
			
			-- Check Frost Fever
			local _s = Disease.State.FF;
			local _r = _s.Time - GetTime(); -- Time Remaining
			if (_s.Active) then
				if (_r > 5 and _state.FFWarned) then
					_state.FFWarned = false;
				elseif (_r <= 5 and not _state.FFWarned) then
					_state.FFWarned = true;
					RuneWatch.Core:ScrollEvent("<"..L["Frost Fever Fading"]..">", unpack(WarningColors[DISEASE_FF]));
				end
			end
		end
	end
end

function Disease:OnInitialize()
	if (not self:IsDeathKnight()) then
		return;
	end

	self.UpdateInterval = 0.5;
	self.UpdateElapsed = 0;
	
	local defaults = {
		profile = {
			warningsEnabled = true,
		}
	}

	Disease.State = {
		BPWarned = false,
		FFWarned = false,
		Diseases = 0,
		BP = {Time = 0, Last = 0, Active = false, id = -1},
		FF = {Time = 0, Last = 0, Active = false, id = -1},
		CF = {Time = 0, Last = 0, Active = false, id = -1},
		EP = {Time = 0, Last = 0, Active = false, id = -1},
		UB = {Time = 0, Last = 0, Active = false, id = -1}
	}
	
	Disease.State.BackTrack = {};
	Disease.State.BackTrack[DISEASE_BP] = Disease.State.BP;
	Disease.State.BackTrack[DISEASE_FF] = Disease.State.FF;
	Disease.State.BackTrack[DISEASE_CF] = Disease.State.CF;
	Disease.State.BackTrack[DISEASE_EP] = Disease.State.EP;
	Disease.State.BackTrack[DISEASE_UB] = Disease.State.UB;
	
	self:Register(Disease, DiseaseConfig.name, DiseaseConfig, defaults);
	Disease.Drag = self:RegisterDrag(Disease.Frame);

	RuneWatch:SaveDefaultDragPosition(self.ModID, 0, 80);
	
	--[===[@alpha@
	if (self.db) then
		self:Print("Database Initialized...");
	else
		self:Print("Database NOT Initialized...");
	end
	--@end-alpha@]===]
	
	-- Initialize Gui
	self:InitGui();
	self:LookupSpellLocale();
	
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("PLAYER_ALIVE");
	self:RegisterEvent("CHARACTER_POINTS_CHANGED");
	
	self:RegisterMessage("RuneWatch_Gui_Update");
	self:RegisterMessage("RuneWatch_Gui_UpdateDrag");
	
	RuneWatch:SendMessage("RuneWatch_Gui_Update");
end

function Disease:CHARACTER_POINTS_CHANGED(event, ...)
	Disease:CheckForEP();
end

function Disease:PLAYER_ALIVE(event, ...)
	Disease:CheckForEP();
end

function Disease:CheckForEP()
	-- Detect the Presence of Ebon Plaguebringer Talent (Note, This will need to be changed any time death-knight talents are adjusted)
	local _EPName,_EPIcon,_EPTier,_EPCol,EPCount = GetTalentInfo(3,27);	
	if (EPCount > 0) then
		HasEP = true;
	else
		HasEP = false;
	end
	
	self:UpdateEP();
end

function Disease:OnEnable()
	self:Print("Enabled...");	
	
	self:UpdateEP();
end

function Disease:UpdateEP()
	-- Adjust CF if EP is present
	if (HasEP) then
		local cf = Disease.Frame.CF;
		
		-- Set the Texture
		cf.Art:SetTexture(WIcons.EP);
		
		cf.Disease = DISEASE_EP;
		--cf.State = Disease.State.EP;
	end
end

function Disease:OnDisable()
	self:Print("Disabled...");
end

function Disease:LookupSpellLocale()
	SpellNames.BP = GetSpellInfo(SpellIDs.BP);
	SpellNames.FF = GetSpellInfo(SpellIDs.FF);
	SpellNames.CF = GetSpellInfo(SpellIDs.CF);
	SpellNames.EP = GetSpellInfo(SpellIDs.EP);
	SpellNames.UB = GetSpellInfo(SpellIDs.UB);
end

function Disease:InitGui()
	self:InitAnchor();
	
	Disease.Frame.BP = self:InitTimer("RW_W_Disease_BP", WIcons.BP, DISEASE_BP);
	Disease.Frame.FF = self:InitTimer("RW_W_Disease_FF", WIcons.FF, DISEASE_FF);
	Disease.Frame.CF = self:InitTimer("RW_W_Disease_CF", WIcons.CF, DISEASE_CF);
	Disease.Frame.UB = self:InitTimer("RW_W_Disease_UB", WIcons.UB, DISEASE_UB);
	
	RuneWatch.RegisterMessage(Disease.Frame.BP, "RuneWatch_Disease_Update");
	RuneWatch.RegisterMessage(Disease.Frame.FF, "RuneWatch_Disease_Update");
	RuneWatch.RegisterMessage(Disease.Frame.CF, "RuneWatch_Disease_Update");
	RuneWatch.RegisterMessage(Disease.Frame.UB, "RuneWatch_Disease_Update");
	
	-- Place Disease Widgets
	Disease.Frame.FF:SetPoint("CENTER", Disease.Frame.Anchor, "CENTER", -40, 0);
	Disease.Frame.BP:SetPoint("CENTER", Disease.Frame.Anchor, "CENTER", 40, 0);
	
	Disease.Frame.CF:SetPoint("CENTER", Disease.Frame.Anchor, "CENTER", -80, 0);
	Disease.Frame.UB:SetPoint("CENTER", Disease.Frame.Anchor, "CENTER", 80, 0);
end

function Disease:InitAnchor()
	local frame = CreateFrame("frame", "RW_Disease", Disease.Frame);
	frame:SetWidth(40);
	frame:SetHeight(40);
	frame:SetPoint("CENTER", Disease.Frame, "CENTER");
	frame:Show();
	
	local bg = frame:CreateTexture("$parent_Frame", "BACKGROUND");
	bg:SetTexture("Interface\\ICONS\\Spell_Nature_NullifyDisease");
	bg:SetVertexColor(0.5, 0.5, 0.5, 1);
	bg:SetAllPoints(frame);
	bg:Show();
	
	frame.Image = bg;
	
	local t = frame:CreateFontString("$parent_Text", "ARTWORK", "SystemFont_Shadow_Huge1");
	t:SetAllPoints(frame);
	t:SetJustifyH("CENTER");
	t:SetJustifyV("MIDDLE");
	t:SetText("0");
	t:Show();
	
	frame.Text = t;
	
	frame:SetScript("OnUpdate", function(self, elapsed) Disease:OnUpdate(self, elapsed) end);
	
	Disease.Frame.Anchor = frame;
end

function Disease:InitTimer(name, iart, disCode)
	local frame = CreateFrame("frame", name, Disease.Frame.Anchor);
	frame:SetWidth(36);
	frame:SetHeight(36);
	frame:SetPoint("CENTER", Disease.Frame.Anchor, "CENTER", 0, 0);
	frame:Show();
	
	local cframe = CreateFrame("cooldown", "$parent_Cool", frame);
	cframe:SetPoint("CENTER", frame, "CENTER");
	cframe:SetWidth(25);
	cframe:SetHeight(25);
	cframe:Show();
	
	-- Omni CC Fix
	cframe.noCooldownCount = true;
	
	cframe:SetFrameLevel(frame:GetFrameLevel() + 1);
	
	frame.Cool = cframe;
	
	local oframe = CreateFrame("frame", "$parent_Over", frame);
	oframe:SetAllPoints(frame);
	oframe:Show();
	oframe:SetFrameLevel(frame:GetFrameLevel() + 2);
	oframe:EnableMouse(true);
	
	frame.Over = oframe;

	-- Lower Art
	
	local art = frame:CreateTexture("$parent_Art", "BACKGROUND");
	art:SetTexture(iart);
	art:SetVertexColor(1, 1, 1, 1);
	art:SetAllPoints(frame);
	art:Show();
	
	frame.Art = art;
	
	-- Over Art
	local ring = oframe:CreateTexture("$parent_Ring", "BACKGROUND");
	ring:SetAllPoints(oframe);
	ring:SetTexture(WIcons.Ring); -- Ring Overlay
	ring:Show();
	
	frame.Ring = ring;
	
	local timer = oframe:CreateFontString("$parent_Timer", "ARTWORK", "RW_TrackerFont");
	timer:SetAllPoints(oframe);
	timer:SetTextColor(1,1,0,0.7);
	timer:SetJustifyH("CENTER");
	timer:SetJustifyV("MIDDLE");
	timer:SetText("0");
	timer:Show();
	
	frame.Timer = timer;
	
	frame.Disease = disCode;
	frame.State = Disease.State.BackTrack[disCode];
	
	function frame:RuneWatch_Disease_Update(event, ...)
		local _disease, _enabled = ...;
		local _state = self.State;
		
		-- Not us? Skip...
		if (self.Disease ~= _disease) then
			return;
		end
		
		local g = self;
		
		if (_enabled) then
			if (g:GetScript("OnUpdate") == nil) then
				g:SetScript("OnUpdate", function(frame, elapsed)
					local _st = frame.State;
					_st.Last = _st.Last + elapsed;
					
					if (_st.Last > 0.5) then
						_st.Last = 0;
						
						-- Do a visual
						local left = floor(_st.Time - GetTime());
						g.Timer:SetText(left);
					end
				end);
			end
			
			g:Show();
		else
			g:Hide();
			
			_state.Active = false;
			
			-- Remove an update script if it's active
			if (g:GetScript("OnUpdate") ~= nil) then
				g:SetScript("OnUpdate", nil);
			end
		end
	end
	
	return frame;
end

--[[=========== Events =========]]--

function Disease:RuneWatch_Gui_Update(self, ...)
	Disease:ScanDebuffs();
end

function Disease:RuneWatch_Gui_UpdateDrag(self, ...)
	local drag = Disease.Drag;
	
	drag:SetParent(Disease.Frame);
	drag:SetPoint("CENTER", Disease.Frame.Anchor, "CENTER");
	drag:SetWidth(60);
	drag:SetHeight(60);
end

function Disease:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local _, ev, src, _, _, dst, _, _, a1, a2, a3, a4, a5, a6, a7, a8 = ...;
	
	if (dst == Disease.TargetGUID) then
		if (ev == "SPELL_AURA_APPLIED" or
		    ev == "SPELL_AURA_REMOVED" or
			ev == "SPELL_AURA_REFRESH") then
			
			local spellID = a1;
			local auraType = a4;
			
			if (auraType == "DEBUFF") then
				Disease:ScanDebuffs();
			end		
		end
	end
end

function Disease:PLAYER_TARGET_CHANGED(event, ...)
	Disease.TargetGUID = UnitGUID("target");
	Disease:ScanDebuffs();
end

--[[========= UPDATE =========]]--

function Disease:ScanBuffs()
	
	local data = { found = false, start = 0, duration = 0, expire = 0 }
	
	for i = 1,40 do
		local nm, rank, icon, count, debuff, duration, expire, isMine, isSteal = UnitAura("player", i, "HELPFUL");
		
		if (nm == "Unholy Blight") then
			data.found = true;
			data.start = expire - duration;
			data.duration = duration;
			data.expire = expire;
			
			return data;
		end
	end

	return data;
end

function Disease:ScanDebuffs()
	
	local dCount = 0;
	local foundBP = false;
	local foundFF = false;
	local foundCF = false;
	local foundEP = false;
	local foundUB = false;
	
	local UBData = Disease:ScanBuffs();
	
	-- Unholy Blight	
	for i = 1,40 do
		local nm, rank, icon, count, debuff, duration, expire, isMine, isSteal = UnitAura("target", i, "HARMFUL");
		
		--if (debuff == "Disease") then
			-- Disease!
			
			if (isMine) then
				if (nm == SpellNames.BP) then
					dCount = dCount + 1;
					-- My Blood Plague
					Disease.State.BP.Time = expire; -- Time the debuff expires
					Disease.State.BP.Active = true;
					
					Disease.Frame.BP.Cool:SetCooldown((expire - duration), duration);
					
					foundBP = true;
				end
				
				if (nm == SpellNames.FF) then
					dCount = dCount + 1;
					-- My Frost Fever
					Disease.State.FF.Time = expire; -- Time the debuff expires
					Disease.State.FF.Active = true;
					
					Disease.Frame.FF.Cool:SetCooldown((expire - duration), duration);
					
					foundFF = true;
				end
				
				if (nm == SpellNames.UB) then
					dCount = dCount + 1;
					-- My Unholy Blight
					
					Disease.State.UB.Time = UBData.expire; -- Time the debuff expires
					Disease.State.UB.Active = true;
					
					Disease.Frame.UB.Cool:SetCooldown((UBData.expire - UBData.duration), UBData.duration);
					
					foundUB = true;
				end
				
				if (nm == SpellNames.CF) then
					dCount = dCount + 1;
					-- My Crypt Fever
					Disease.State.CF.Time = expire; -- Time the debuff expires
					Disease.State.CF.Active = true;
					
					Disease.Frame.CF.Cool:SetCooldown((expire - duration), duration);
					
					foundCF = true;
				end
				
				if (nm == SpellNames.EP) then
					dCount = dCount + 1;
					-- My Ebon Plague
					Disease.State.CF.Time = expire; -- Time the debuff expires
					Disease.State.CF.Active = true;
					
					Disease.Frame.CF.Cool:SetCooldown((expire - duration), duration);
					
					foundCF = true;
					foundEP = true;
				end
			end
		--end
	end
	
	-- Update Units
	RuneWatch:SendMessage("RuneWatch_Disease_Update", DISEASE_BP, foundBP);
	RuneWatch:SendMessage("RuneWatch_Disease_Update", DISEASE_FF, foundFF);
	
	if (not HasEP) then
		RuneWatch:SendMessage("RuneWatch_Disease_Update", DISEASE_CF, foundCF);
	else
		RuneWatch:SendMessage("RuneWatch_Disease_Update", DISEASE_EP, foundEP);
	end
	
	RuneWatch:SendMessage("RuneWatch_Disease_Update", DISEASE_UB, foundUB);
		
	-- Update Diseases
	local _f = Disease.Frame.Anchor;
	_f.Text:SetText(dCount);
end
