--[[
  Title: RuneWatch v2.0.0 Beta 5: Disease Tracker
  Author: voorije
  Revised: 2009-01-26T14:55:47Z
--]]

local AceDB = LibStub("AceDB-3.0")
local AceLocale = LibStub("AceLocale-3.0")

-- Find RuneWatch Object
assert(RuneWatch, "Unable to find RuneWatch Object.");

-- Find Ace3
--assert(AceAddon, "Unable to load AceAddon-3.0");
assert(AceDB, "Unable to load AceAddon-3.0");
assert(AceLocale, "Unable to load AceLocale-3.0");

local L = AceLocale:GetLocale("RuneWatch_Procs");

local ProcsConfig = {
	type = "group",
	name = "Procs & States",
	desc = "Procs & States",
	args = {}
}

local Combat = LibStub("LibCombatDetails-1.0");
if not Combat then error("RuneWatch_Procs requires LibCombatDetails-1.0"); end

local Procs = RuneWatch:NewModule("Procs", RuneWatch.ModuleBase, "AceConsole-3.0", "AceEvent-3.0")
Procs.Frame = CreateFrame("frame", "RuneWatch_Procs", RuneWatchFrame);
Procs.Frame:SetPoint("CENTER", UIParent, "CENTER", 100, 0);
Procs.Frame:SetWidth(10);
Procs.Frame:SetHeight(10);
Procs.Frame:Show();
Procs.Frame:SetMovable(true);
Procs.Frame:SetScript("OnUpdate", function(self, elapsed) Procs:OnUpdate(self, elapsed); end)

do
	Procs.Talents = {}
	
	local function RegisterTalent(id, page, num)
		local entry = {
			-- Details
			ID = id,
			Page = page,
			Number = num,
			
			-- State
			Has = false,
			Icon = "",
		}
		
		Procs.Talents[id] = entry;
	end
	
	-- Talents to Scan
	
	RegisterTalent("T_SuddenDoom", 1, 21)
end

do
	Procs.ProcList = {}
	
	local function RegisterProc(id, typ, icon, title, meta)
		local entry = {
			ID = id,
			Type = typ,
			Icon = icon,
			Title = title,
			Meta = meta
		}
		
		Procs.ProcList[id] = entry	
	end
	
	local function RegisterCombatAura(id, title, spellID)
		if (type(spellID) == "table") then
			_sid = spellID[1]
			_stb = spellID
		else
			_sid = spellID
			_stb = { spellID }
		end
	
		--Procs:Print("Getting Spell Info: "..(_sid or "nil"))
	
		local sName, sRank, sIcon = GetSpellInfo(_sid)
	
		if title == nil then
			title = sName
		end
	
		local meta = {
			SpellID = _stb,
			SpellName = sName,
		}
		
		RegisterProc(id, "CAura", sIcon, title, meta)
	end
	
	
	RegisterCombatAura("CA_SuddenDoom", L["Free Death Coil"], {50466} ) -- Sudden Doom / Death Trance Proc
	--RegisterCombatAura("CA_BloodyVeng", L["Increased Damage"], {50447, 50448, 50449} ) -- Bloody Veng / +1-3% Damage after crit
	--RegisterCombatAura("CA_SpellDeflect", L["Spell Deflection"], {51911} ) -- Spell Deflection
	RegisterCombatAura("CA_BladeBarrier", L["Increased Parry"], {51789} ) -- Increased Parry
	
	-- Frost
	RegisterCombatAura("CA_KillingMachine", nil, {51124} ) -- Killing Machine
	RegisterCombatAura("CA_Rime", L["Free Howling Blast"], {59052} ) -- Rime / Free Howling Blast
	RegisterCombatAura("CA_Acclimation", L["Spell Resistance"], {50486, 50362, 50485, 50488, 50489, 50490} ) -- Acclimation (Displays Arcane Icon)
	
	
end

do
	Procs.CoolList = { }
	
	local function RegisterCooldown(id, spellID, doAlmost)
		local entry = {
			-- Cooldown Details
			SpellID = spellID,
			Name = "",
			Icon = "",
			-- Options
			DoAlmost = doAlmost,
			-- States
			Cache = {
				Start = 0,
				Duration = 0,
			},
			Almost = false,
			Enabled = false,
		}
		
		Procs.CoolList[id] = entry;
	end
	
	-- Cooldown registrations
	
	-- With Almost Warning (Combat Abilities)
	RegisterCooldown("SP_DeathAndDecay", 43265, true)
	RegisterCooldown("SP_DeathGrip", 49576, true)
	RegisterCooldown("SP_DancingRW", 49028, true)
	RegisterCooldown("SP_Strangulate", 47476, true)
	RegisterCooldown("SP_VampBlood", 55233, true)
	
	-- Frost
	RegisterCooldown("SP_UnbreakableArmor", 51271, true)
	RegisterCooldown("SP_Lichborne", 49039, true)
	RegisterCooldown("SP_Deathchill", 49796, true)
	RegisterCooldown("SP_HungeringCold", 49203, true)
	
	-- Unholy
	RegisterCooldown("SP_AntiMagicZone", 51052, true)
	RegisterCooldown("SP_BoneShield", 49222, true)
	RegisterCooldown("SP_CorpseExplosion", 49158, false) -- False cuz cooldown is short
	RegisterCooldown("SP_SummonGargoyle", 49206, true)
	
	-- Without Almost Warning (Non-Combat Abilities / Short Cooldowns)
	RegisterCooldown("SP_RaiseDead", 46584, false) -- Not a pressing in-combat action, no almost warning
	RegisterCooldown("SP_MindFreeze", 47528, false) -- Short Cooldown, no need for almost warning.
	RegisterCooldown("SP_DeathGate", 50977, false) -- Non-Combat
	
	
end

local function CacheCooldown(id)
	local entry = Procs.CoolList[id]
	
	entry.Cache.Start = GetTime()
	entry.Cache.Duration = -1 -- Redundant for now, may be usefull later for adjusting
	
	--Procs:Print("[COOL]: Cast Detected '"..id.."' = "..entry.Cache.Start);
end

local function CacheDuration(id)
	local now = GetTime()
	local entry = Procs.CoolList[id]
	
	if (entry.Cache.Duration ~= -1) then
		return true
	end
	
	if (now > entry.Cache.Start + 0.5) then -- If some time has passed, GetSpellInfo(...) should be updated
		local s, d, e = GetSpellCooldown(entry.Name)--, BOOKTYPE_SPELL)
		entry.Cache.Duration = d;
		
		--Procs:Print("[COOL]: Cooldown Info '"..id.."|"..entry.Name.."' = "..(s or "nil").."|"..(d or "nil").."|"..(e or "nil"));
		
		return true
	end
	
	--Procs:Print("[COOL]: Not Ready '"..id);
	
	-- Not ready
	return false
end

local function ClearCooldown(id)
	--Procs:Print("[COOL]: Clearing '"..id.."'")
	local entry = Procs.CoolList[id]
	
	entry.Cache.Start = 0
	entry.Cache.Duration = -1
end

function Procs:OnInitialize()
	if (not self:IsDeathKnight()) then
		return;
	end

	local defaults = {
		profile = {
			colors = {
				coolReady = { r = 1.0, g = 0.3, b = 0.5, a = 1.0 },
				coolAReady = { r = 1.0, g = 0.5, b = 0.8, a = 1.0 },
			}
		}
	}
	
	self:Register(Procs, ProcsConfig.name, ProcsConfig, defaults);
	Procs.Drag = self:RegisterDrag(Procs.Frame);

	RuneWatch:SaveDefaultDragPosition(self.ModID, 100, 0);
	
	--[===[@alpha@
	if (self.db) then
		self:Print("Database Initialized...");
	else
		self:Print("Database NOT Initialized...");
	end
	--@end-alpha@]===]
	
	self:ScanSpells();
	
	self:InitGui();
	
	Procs:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	Procs:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	Procs:RegisterEvent("CHARACTER_POINTS_CHANGED");
	Procs:RegisterEvent("PLAYER_ALIVE");
	
	self:RegisterMessage("Cooldown_Ready"); -- Ability Cooled
	self:RegisterMessage("Cooldown_Ready_Almost"); -- Ability 5 seconds from cooling
	
	self:RegisterMessage("RuneWatch_Gui_Update");
	self:RegisterMessage("RuneWatch_Gui_UpdateDrag");
end

function Procs:RuneWatch_Gui_Update(self, ...)
	
end

function Procs:RuneWatch_Gui_UpdateDrag(self, ...)
	local Drag = Procs.Drag;
		
	Drag:SetParent(Procs.Frame.Center);
	Drag:ClearAllPoints();
	Drag:SetPoint("LEFT", Procs.Frame.Center, "LEFT", -10, 0);
	Drag:SetWidth(160);
	Drag:SetHeight(100);
	return
end


function Procs:OnEnable()
	self:Print("Enabled...");
	RuneWatch:SendMessage("RuneWatch_Gui_Update");
end

function Procs:OnDisable()
	self:Print("Disabled...");
end

local CooldownScanInterval = 0.1;
local CooldownScanElapsed = 0
function Procs:OnUpdate(self, elapsed)
	CooldownScanElapsed = CooldownScanElapsed + elapsed;
	
	if (CooldownScanElapsed >= CooldownScanInterval) then
		CooldownScanElapsed = 0
		
		Procs:ScanCooldowns()
	end
end

function Procs:PLAYER_ALIVE(event, ...)
	Procs:ScanTalents();
end

function Procs:CHARACTER_POINTS_CHANGED(event, ...)
	Procs:ScanTalents();
end

function Procs:UNIT_SPELLCAST_SUCCEEDED(event, ...)
	local unit, spellName = ...
	
	if (unit == "player") then
		for k,v in pairs(Procs.CoolList) do
			if (v.Name == spellName) then
				-- Found our Proc
				Procs.CoolList[k].Enabled = true
				Procs.CoolList[k].Almost = false
				CacheCooldown(k);
				return
			end
		end
	end
end

function Procs:ScanTalents()
	local _talents = Procs.Talents;
	
	for k,v in pairs(_talents) do
		local tName, tPath, _, _, tRank = GetTalentInfo(v.Page, v.Number)
		
		_talents[k].Icon = tPath
		
		if (tRank > 0) then
			_talents[k].Has = true
		else
			_talents[k].Has = false
		end
	end
end

function Procs:ScanSpells()
	local cools = Procs.CoolList
	
	for k,v in pairs(cools) do
		local sName, _, sIcon = GetSpellInfo(v.SpellID)
		
		cools[k].Name = sName;
		cools[k].Icon = sIcon;
	end
end

--- Responsible for scanning the cooldown of each cooldown we're watching and firing the 
-- event linked to the cooldown.
function Procs:ScanCooldowns()
	local cools = Procs.CoolList
	
	for k,v in pairs(cools) do
		if (v.Enabled) then
			local ready = CacheDuration(k);
		
			if (ready) then
				local remaining = cools[k].Cache.Duration - (GetTime() - cools[k].Cache.Start)
			
				if remaining <= 0 then
					-- Cooldown Ready
					ClearCooldown(k);
					cools[k].Enabled = false
					cools[k].Almost = false
					Procs:SendMessage("Cooldown_Ready", v, cStart, cDuration, cEnabled);
				elseif remaining <= 5 then
					if (not v.Almost) then
						-- Fire Almost Ready Event
						cools[k].Almost = true
						Procs:SendMessage("Cooldown_Ready_Almost", v, cStart, cDuration, cEnabled);
					end
				else
					-- remaining > 5
					if (v.Almost) then
						cools[k].Almost = false
					end
				end
			end
		end
	end
end

function Procs:InitGui()
	self:GUI_CreateCoolCenter();
	self:GUI_CreateProcBars();
end

function Procs:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	--self:Print("C_EVENT: "..(Combat:GetEvent(...)));
	if (Combat:IsAura(...)) then
		local id, nm = Combat:GetSpellDetails(...);
		local _pl = Procs.ProcList;
		
		for k,v in pairs(_pl) do
			if (v.Type == "CAura") then
				local _stb = v.Meta.SpellID
				
				for i = 1,#_stb do
					local _s = _stb[i]
					
					if (_s == id) then
						-- Found
						Procs:CombatAura_Event(k, ...)
						return
					end
				end
			end
		end
	end
end

function Procs:CombatAura_Event(id, ...)
	if (Combat:IsAuraApplied(...)) then
		Procs:CombatAura_Applied(id, ...)
	elseif (Combat:IsAuraRefresh(...)) then
		Procs:CombatAura_Refresh(id, ...)
	elseif (Combat:IsAuraRemoved(...)) then
		Procs:CombatAura_Removed(id, ...)
	end
end

function Procs:CombatAura_Applied(id, ...)
	local v = Procs.ProcList[id]
	
	self:PROC_Register(v);
end

function Procs:CombatAura_Refresh(id, ...)
	--local v = Procs.ProcList[id]
	
	-- Do Nothing
end

function Procs:CombatAura_Removed(id, ...)
	local v = Procs.ProcList[id]
	
	self:PROC_Unregister(v);
end

function Procs:Cooldown_Ready(event, ...)
	local cool, cStart, cDuration, cEnabled = ...;
	
	-- Trigger Ability Flash
	Procs.Frame.Center:FlashIcon(cool.Icon);
	
	local message = "< "..cool.Name.." "..L["Ready"].." >";
	local color = Procs.db.colors.coolReady
	RuneWatch.Core:ScrollEvent(message, color.r, color.g, color.b)
end

function Procs:Cooldown_Ready_Almost(event, ...)
	local cool, cStart, cDuration, cEnabled = ...;
	
	if (not cool.DoAlmost) then
		return
	end
	
	local message = "< "..cool.Name.." "..L["Almost Ready"].." >";
	local color = Procs.db.colors.coolAReady
	RuneWatch.Core:ScrollEvent(message, color.r, color.g, color.b)
end
