assert(RuneWatch, "RuneWatch Not Loaded.");

local Procs = RuneWatch:GetModule("Procs");
assert(Procs, "Procs Module not loaded.");

local ProcRegistry = {
	[1] = { ProcDetails = nil, Visible = false, Icon = "", Title = "Empty Proc Bar" },
	[2] = { ProcDetails = nil, Visible = false, Icon = "", Title = "Empty Proc Bar" },
	[3] = { ProcDetails = nil, Visible = false, Icon = "", Title = "Empty Proc Bar" }
}

local ProcQueue = {}

local function PROC_SetRegistry(id, proc)
	ProcRegistry[id].ProcDetails = proc;
	ProcRegistry[id].Visible = true;
	ProcRegistry[id].Icon = proc.Icon
	ProcRegistry[id].Title = proc.Title
	
	-- Update Actual GUI Element
	local _ui = Procs.Frame.PBars[id]
	_ui:Display(proc.Title, proc.Icon)
end

local function PROC_ResetRegistry(id)
	local _l = ProcRegistry[id]
	ProcRegistry[id].ProcDetails = nil
	ProcRegistry[id].Visible = false
	ProcRegistry[id].Icon = ""
	ProcRegistry[id].Title = ""
	
	local _ui = Procs.Frame.PBars[id]
	_ui:Hide()
end

local function PROC_ReSort()
	local _empty = 0
	for i = 1,3 do
		if (ProcRegistry[i].ProcDetails == nil) then
			_empty = i
			break
		end
	end
	
	if (_empty > 0) then
		if #ProcQueue > 0 then
			-- Items to be moved into the system
			local _qitem = ProcQueue[1]
			table.remove(ProcQueue, 1)
			
			PROC_Register(_empty, _qitem)
		end
	end
end

function Procs:PROC_Register(proc)
	for i = 1,3 do
		if (ProcRegistry[i].ProcDetails == nil) then
			-- Can Register
			PROC_SetRegistry(i, proc)
			return
		end
	end
	
	-- If we made it here, we need to queue the item
	table.insert(ProcQueue, proc)
end

function Procs:PROC_Unregister(proc)
	for i = 1,3 do
		if (ProcRegistry[i].ProcDetails ~= nil) then
			if (ProcRegistry[i].ProcDetails.ID == proc.ID) then
				PROC_ResetRegistry(i)
				PROC_ReSort()
				return
			end
		end
	end
	
	for i = 1,#ProcQueue do
		if (ProcQueue[i].ID == proc.ID) then
			table.remove(ProcQueue, i)
			return
		end
	end
	
	-- Wasn't Present, Do nothing
end

function Procs:GUI_CreateProcBars()
	Procs.Frame.PBars = {}
	
	local _b = Procs.Frame
	
	if (Procs.Frame.Center) then
		_b = Procs.Frame.Center
	end
	
	local _pb = Procs.Frame.PBars
	
	_pb[1] = self:GUI_CreateProcBar(1)
	_pb[2] = self:GUI_CreateProcBar(2)
	_pb[3] = self:GUI_CreateProcBar(3)
	
	_pb[2]:SetPoint("LEFT", _b, "LEFT", 10, -40)
	_pb[3]:SetPoint("LEFT", _b, "LEFT", 10, 40)
	_pb[1]:SetPoint("LEFT", _b, "RIGHT", 5, 0)
end

function Procs:GUI_CreateProcBar(id)
	local _base = UIParent
	
	if (Procs.Frame.Center) then
		_base = Procs.Frame.Center
	end

	local frame = CreateFrame("frame", "RW_Procs_PBar"..id, UIParent)
	frame:SetWidth(150)
	frame:SetHeight(26)
	frame:Hide()
	
	local tex = frame:CreateTexture("$parent_Icon", "ARTWORK")
	tex:SetPoint("LEFT", frame, "LEFT", 5, 0)
	tex:SetWidth(20)
	tex:SetHeight(20)
	tex:SetTexture(1,0,0,0.5)
	tex:Show()
	
	local text = frame:CreateFontString("$parent_Title", "ARTWORK", "SystemFont_Outline_Small")
	text:SetPoint("RIGHT", frame, "RIGHT", -5, 0)
	text:SetPoint("LEFT", tex, "RIGHT", 10, 0)
	text:SetHeight(20)
	text:SetText("Example Text...")
	text:SetJustifyH("LEFT")
	text:Show()
	
	frame.ID = id
	frame.Icon = tex
	frame.Title = text
	
	function frame:Display(title, icon)
		frame.Icon:SetTexture(icon)
		frame.Title:SetText(title)
		
		frame:Show()
	end
	
	return frame
end

local function ANIM_CoolCenterFlash(self, elapsed)
	self.FElapsed = self.FElapsed + elapsed;
	
	if (self.FElapsed < 1) then
		self.Texture:SetAlpha(self.FElapsed)
	elseif(self.FElapsed < 2) then
		local t = 1 - (self.FElapsed - 1)
		self.Texture:SetAlpha(t)
	else
		self.Texture:Hide()
		self:SetScript("OnUpdate", nil) -- Kill this script
	end
end

function Procs:GUI_CreateCoolCenter()
	local CenterTextureSize = 32

	local frame = CreateFrame("frame", "$parent_Center", Procs.Frame);
	frame:SetPoint("CENTER", Procs.Frame, "CENTER")
	frame:SetWidth(CenterTextureSize + 14)
	frame:SetHeight(CenterTextureSize + 14)
	frame:Show();
	
	frame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
		tile = false, tileSize = 0, edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 }
	});
	frame:SetBackdropColor(0,0,0,0.5)
	
	local tex = frame:CreateTexture("$parent_Tex", "ARTWORK")
	tex:SetPoint("CENTER", frame, "CENTER")
	tex:SetHeight(CenterTextureSize);
	tex:SetWidth(CenterTextureSize)
	tex:Hide()
	
	frame.Texture = tex;
	
	function frame:FlashIcon(texture)
		local tex = self.Texture;
		
		tex:Show()
		tex:SetAlpha(0)
		tex:SetTexture(texture)
		
		frame.FElapsed = 0
		
		frame:SetScript("OnUpdate", function(self, elapsed) 
			ANIM_CoolCenterFlash(self, elapsed)
		end);
	end
	
	Procs.Frame.Center = frame;
end
