--[[
  Title: RuneWatch v2.0.0 Beta 5: Base Skin
  Author: voorije
  Revised: 2009-01-22T17:46:02Z
--]]

local AceAddon = LibStub("AceAddon-3.0")
local AceDB = LibStub("AceDB-3.0")

-- Find RuneWatch Object
assert(RuneWatch, "Unable to find RuneWatch Object.");

-- Find Ace3
assert(AceAddon, "Unable to load AceAddon-3.0");
assert(AceDB, "Unable to load AceAddon-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("RuneWatch_Runes")

local Base = RuneWatch:NewModule("Base", RuneWatch.ModuleBase, "AceConsole-3.0", "AceEvent-3.0")
Base.Frame = CreateFrame("frame", "RuneWatch_Base", RuneWatchFrame);
Base.Frame:SetPoint("CENTER", UIParent, "CENTER");
Base.Frame:SetWidth(10);
Base.Frame:SetHeight(10);
Base.Frame:Show();
Base.Frame:SetMovable(true);

local SkinArt = 
{
	Base = 
	{
		Generic = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Orb-Generic",
		Frost = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Orb-Frost",
		Unholy = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Orb-Unholy",
		Blood = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Orb-Blood",
	},
	Classic = 
	{
		Base = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Classic-Base",
		Bar = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Classic-Bar",
		BarBorder = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Classic-BarBorder",
	}
}

local BaseConfig = {
	type = "group",
	name = L["Rune Frame"],
	desc = L["Rune Frame"],
	args = {
		header_visual = {
			order = 1,
			type = "header",
			name = L["Visual Appearance"]
		},
		skin = {
			order = 5,
			type = "select",
			name = L["Skin"],
			desc = L["Skin, changes the appearance of the rune frame."],
			values = { Compact = "Compact Theme", Classic = "Classic Theme" },
			get = function(info) return Base.db.skin; end,
			set = function(info,v) Base.db.skin = v; RuneWatch:SendMessage("RuneWatch_Gui_Update"); end,
			style = "dropdown",
			width = "full",
		},
		theme = {
			order = 10,
			type = "select",
			name = L["Theme"],
			desc = L["Skin Theme, changes the inner orb."],
			values = { Generic = "Generic Theme", Blood = "Blood Theme", Frost = "Frost Theme", Unholy = "Unholy Theme" },
			get = function(info) return Base.db.theme; end,
			set = function(info, v) Base.db.theme = v; RuneWatch:SendMessage("RuneWatch_Gui_Update"); end,
			style = "dropdown",
			width = "full"
		}
	}
}

function Base:OnInitialize()
	if (not self:IsDeathKnight()) then
		return;
	end

	local defaults = {
		profile = {
			skin = "Compact",
			theme = "Generic"
		}
	}

	self:Register(Base, BaseConfig.name, BaseConfig, defaults);
	Base.Drag = self:RegisterDrag(Base.Frame);

	--[===[@alpha@
	if (self.db) then
		self:Print("Database Initialized...");
	else
		self:Print("Database NOT Initialized...");
	end
	--@end-alpha@]===]
	
	-- Initialize Gui
	self:InitCompactGui();
	self:InitClassicGui();
	self:RegisterMessage("RuneWatch_Gui_Update");
	self:RegisterMessage("RuneWatch_Gui_UpdateDrag");
	
	self:RegisterMessage("RuneWatch_UnitData_RunicPower");
end

function Base:InitConfig()
	
end

function Base:OnEnable()
	self:Print("Enabled...");
	RuneWatch:SendMessage("RuneWatch_Gui_Update");
end

function Base:OnDisable()
	self:Print("Disabled...");
end

function Base:RuneWatch_Gui_Update(self, ...)
	
	--[===[@alpha@
	Base:Print("Caught GUI Update Event...");
	--@end-alpha@]===]
	
	Base.Frame.Compact:Hide();
	Base.Frame.Classic:Hide();
		
	if (Base.db.skin == "Compact") then
		Base.Frame.Compact:Show();
		Base:UpdateCompactGui();
	end
	
	if (Base.db.skin == "Classic") then
		Base.Frame.Classic:Show();
		Base:UpdateClassicGui();
	end
	
end

function Base:RuneWatch_Gui_UpdateDrag(self, ...)
	
	--[===[@alpha@
	Base:Print("Caught GUI Drag Update Event...");
	--@end-alpha@]===]
	
	if (Base.db.skin == "Compact") then
		local Drag = Base.Drag;
		
		Drag:SetParent(Base.Frame.Compact);
		Drag:ClearAllPoints();
		Drag:SetPoint("CENTER", Base.Frame.Compact, "CENTER");
		Drag:SetWidth(150);
		Drag:SetHeight(150);
		return
	end
	
	if (Base.db.skin == "Classic") then
		local Drag = Base.Drag;
		
		Drag:SetParent(Base.Frame.Classic);
		Drag:ClearAllPoints();
		Drag:SetPoint("CENTER", Base.Frame.Classic, "CENTER", 0, -25);
		Drag:SetWidth(300);
		Drag:SetHeight(150);
		
		return
	end
	
	--[===[@alpha@
	Base:Print("[ERROR]: No Skin updated the drag frame!");
	--@end-alpha@]===]
end

--[================= COMPACT SKIN ============]--

local CompactMountData = {
	parent = Base.Frame,
	runes = {}
}

function Base:InitCompactGui()
	local frame = CreateFrame("frame", "$parent_Compact", Base.Frame);
	frame:SetFrameStrata("LOW");
	frame:SetFrameLevel(0);
	frame:SetWidth(150);
	frame:SetHeight(150);
	frame:SetPoint("CENTER", Base.Frame, "CENTER");
	frame:Show();
	
	local art = frame:CreateTexture("$parent_T_Back", "BACKGROUND");
	art:SetPoint("CENTER", frame, "CENTER");
	art:SetWidth(150);
	art:SetHeight(150);
	art:Show();
	
	frame.Art = art;
	
	local rp = frame:CreateFontString("$parent_F_RP", "ARTWORK", "SystemFont_OutlineThick_Huge4");
	rp:SetText("0");
	rp:SetTextColor(1,0.5,0,1);	

	rp:SetPoint("CENTER", frame, "CENTER", 0, 0)
	rp:SetWidth(70);
	rp:SetHeight(70);
	rp:SetJustifyH("CENTER");
	rp:SetJustifyV("MIDDLE");
	
	frame.Counter = rp;
	
	Base.Frame.Compact = frame;
	Base.Frame:Show();
	
	CompactMountData.parent = frame;
	CompactMountData.runes = {
		[1] = {"CENTER", Base.Frame.Compact, "CENTER", -45, 0},
		[2] = {"CENTER", Base.Frame.Compact, "CENTER", -30, -35},
		[3] = {"CENTER", Base.Frame.Compact, "CENTER", -24, 35},
		[4] = {"CENTER", Base.Frame.Compact, "CENTER", 24, 35},
		[5] = {"CENTER", Base.Frame.Compact, "CENTER", 30, -35},
		[6] = {"CENTER", Base.Frame.Compact, "CENTER", 45, 0},
	}
end

function Base:UpdateCompactGui()
	local artTexture = SkinArt.Base[Base.db.theme] or SkinArt.Base.Generic;
	local Compact = Base.Frame.Compact;
	Compact.Art:SetTexture(artTexture);
	
	RuneWatch:SendMessage("RuneWatch_Gui_RuneMount", CompactMountData);
end

--[================= CLASSIC SKIN ============]--

local ClassicMountData = {
	parent = Base.Frame,
	runes = {}
}

function Base:InitClassicGui()
	local frame = CreateFrame("frame", "$parent_Classic", Base.Frame);
	frame:SetFrameStrata("LOW");
	frame:SetFrameLevel(5);
	frame:SetWidth(150);
	frame:SetHeight(150);
	frame:SetPoint("CENTER", Base.Frame, "CENTER");
	frame:Show();
	
	local frameBase = CreateFrame("frame", "$parent_Base", frame);
	frameBase:SetFrameStrata("LOW");
	frameBase:SetFrameLevel(3);
	frameBase:SetWidth(300);
	frameBase:SetHeight(150);
	frameBase:SetPoint("CENTER", Base.Frame, "CENTER", 0, -25);
	frameBase:Show();
	
	local fbArt = frameBase:CreateTexture("$parent_T_Back", "BACKGROUND");
	fbArt:SetAllPoints(frameBase);
	fbArt:Show();
	local fbBar = frameBase:CreateTexture("$parent_T_Bar", "BORDER");
	fbBar:SetPoint("LEFT", fbArt, "LEFT");
	fbBar:SetHeight(150);
	fbBar:SetWidth(300);
	fbBar:Show();
	
	function fbBar:SetPercent(perc)
		--Base:Print("W: "..(300*perc).." P: "..perc);
	
		self:SetWidth(300*perc);
		self:SetTexCoord(0, perc, 0, 1);
	end
	
	local fbBord = frameBase:CreateTexture("$parent_T_BarBorder", "ARTWORK");
	fbBord:SetAllPoints(frameBase);
	fbBord:Show();
	
	frame.Base = frameBase;
	frameBase.Art = fbArt;
	frameBase.Bar = fbBar;
	frameBase.Border = fbBord;
	
	frame.Bar = fbBar; -- Shortcut
	
	local art = frame:CreateTexture("$parent_T_Back", "BACKGROUND");
	art:SetPoint("CENTER", frame, "CENTER");
	art:SetWidth(150);
	art:SetHeight(150);
	art:Show();
	
	frame.Art = art;
	
	local rp = frame:CreateFontString("$parent_F_RP", "ARTWORK", "SystemFont_OutlineThick_Huge4");
	rp:SetText("0");
	rp:SetTextColor(1,0.5,0,1);	

	rp:SetPoint("CENTER", frame, "CENTER", 0, 0)
	rp:SetWidth(70);
	rp:SetHeight(70);
	rp:SetJustifyH("CENTER");
	rp:SetJustifyV("MIDDLE");
	
	frame.Counter = rp;
	
	Base.Frame.Classic = frame;
	Base.Frame:Show();
	
	ClassicMountData.parent = frame;
	ClassicMountData.runes = {
		[1] = {"CENTER", Base.Frame.Classic, "CENTER", -110, -23},
		[2] = {"CENTER", Base.Frame.Classic, "CENTER", -70, -57},
		[3] = {"CENTER", Base.Frame.Classic, "CENTER", -24, -75},
		[4] = {"CENTER", Base.Frame.Classic, "CENTER", 24, -75},
		[5] = {"CENTER", Base.Frame.Classic, "CENTER", 70, -57},
		[6] = {"CENTER", Base.Frame.Classic, "CENTER", 110, -23},
	}
end

function Base:UpdateClassicGui()
	local artTexture = SkinArt.Base[Base.db.theme] or SkinArt.Base.Generic;
	local baseTexture = SkinArt.Classic.Base;
	
	local Classic = Base.Frame.Classic;
	Classic.Art:SetTexture(artTexture);
	Classic.Base.Art:SetTexture(SkinArt.Classic.Base);
	Classic.Base.Bar:SetTexture(SkinArt.Classic.Bar);
	Classic.Base.Border:SetTexture(SkinArt.Classic.BarBorder);
	
	RuneWatch:SendMessage("RuneWatch_Gui_RuneMount", ClassicMountData);
end

function Base:RuneWatch_UnitData_RunicPower(self, ...)
	local rp, rpmax = ...;
	
	if (Base.db.skin == "Compact") then
		Base.Frame.Compact.Counter:SetText(rp);
	end
	
	if (Base.db.skin == "Classic") then
		local v = rp/rpmax;
		Base.Frame.Classic.Counter:SetText(rp);
		
		if (v ~= 0) then
			Base.Frame.Classic.Bar:SetPercent(v);
		else
			Base.Frame.Classic.Bar:SetPercent(0.01);
		end
	end
end
