--[[
  Title: RuneWatch v2.0.0 Beta 5: Runes & Runic Power
  Author: voorije
  Revised: 2009-01-22T17:46:02Z
--]]

local AceAddon = LibStub("AceAddon-3.0")
local AceDB = LibStub("AceDB-3.0")

-- Find RuneWatch Object
assert(RuneWatch, "Unable to find RuneWatch Object.");

-- Find Ace3
assert(AceAddon, "Unable to load AceAddon-3.0");
assert(AceDB, "Unable to load AceAddon-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("RuneWatch_Runes")

local Runes = RuneWatch:NewModule("Runes", RuneWatch.ModuleBase, "AceConsole-3.0", "AceEvent-3.0")

local RuneTextures = {
	Blizzard = {
		[1] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Blood",
		[2] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Unholy",
		[3] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Frost",
		[4] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Death"
	},
	Custom1 = {
		[1] = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Blood",
		[2] = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Unholy",
		[3] = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Frost",
		[4] = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Death"
	},
	Custom2 = {
		[1] = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Blood2",
		[2] = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Unholy2",
		[3] = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Frost2",
		[4] = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Death2"
	}
}

local BorderTextures = {
	Blizzard = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Ring",
	Custom1 = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Border",
	Custom2 = "Interface\\AddOns\\RuneWatch_Runes\\Artwork\\Rune-Border",
}

local RuneConfig = {
	type = "group",
	name = L["Runes"],
	desc = L["Runes Configuration"],
	args = {
		header_visual = {
			order = 1,
			type = "header",
			name = L["Visual Appearance"]
		},
		theme = {
			order = 5,
			type = "select",
			name = L["Rune Theme"],
			desc = L["Select the visual theme of the runes."],
			values = { Blizzard = L["Blizzard Runes"], Custom1 = L["RuneWatch Theme 1"], Custom2 = L["RuneWatch Theme 2"] },
			get = function(info) return Runes.db.skin; end,
			set = function(info,v) Runes.db.skin = v; RuneWatch:SendMessage("RuneWatch_Gui_Update"); end,
			style = "dropdown",
			--width = "full"
		},
		rune_order = {
			order = 10,
			type = "select",
			name = L["Rune Order"],
			desc = L["Select the order of the runes"],
			values = {
				buf = L["Blood, Unholy, Frost"],
				bfu = L["Blood, Frost, Unholy"],
				fbu = L["Frost, Blood, Unholy"],
				fub = L["Frost, Unholy, Blood"],
				ubf = L["Unholy, Blood, Frost"],
				ufb = L["Unholy, Frost, Blood"],
			},
			get = function(info) return Runes.db.order end,
			set = function(info,v) Runes.db.order = v; RuneWatch:SendMessage("RuneWatch_Gui_Update"); end,
		},
		rune_timer = {
			order = 15,
			type = "group",
			name = L["Rune Timer"],
			desc = L["Rune Timer"],
			inline = true,
			args = {
				dispNumerical = {
					order = 1,
					type = "toggle",
					name = L["Display Numerical Timer"],
					desc = L["Display / Hide the Numerical Timer for runes."],
					width = "full",
					get = function(info) return Runes.db.timer.numerical; end,
					set = function(info, v) Runes.db.timer.numerical = v; Runes:RuneWatch_Gui_Update("RuneWatch_Gui_Update", nil); end,
				},
				--[[dispCooldown = {
					order = 2,
					type = "toggle",
					name = "Display Cooldown Timer",
					desc = "Display / Hide the Cooldown (radial) Timer for runes.",
					width = "full",
					get = function(info) return Runes.db.timer.cooldown; end,
					set = function(info, v) Runes.db.timer.cooldown = v; Runes:RuneWatch_Gui_Update("RuneWatch_Gui_Update", nil); end,
				},]]--
				numFontSize = {
					order = 3,
					type = "range",
					width = "full",
					name = L["Numerical Timer Font-Size"],
					desc = L["Sets the font size of the numerical timer display."],
					min = 6,
					max = 24,
					step = 1,
					get = function(info) return Runes.db.timer.fontSize; end,
					set = function(info,v) Runes.db.timer.fontSize = v; Runes:UpdateFont(); end,
				},
				colorDesc = {
					order = 10,
					type = "description",
					name = L["Rune Timer colors, below you set the colors of the text for the rune-timers. Remember you want to choose colors that stand out overtop of the rune's color."],
				},
				blood = {
					order = 11,
					type = "color",
					name = L["Blood Runes"],
					desc = L["Blood Runes"],
					hasAlpha = true,
					get = function(info) local v = Runes.db.colors.blood; return v.r, v.g, v.b, v.a; end,
					set = function(info, re,gr,bl,al) Runes.db.colors.blood = { r = re, g = gr, b = bl, a = al }; Runes:RuneWatch_Gui_Update("RuneWatch_Gui_Update", "null") end
				},
				unholy = {
					order = 12,
					type = "color",
					name = L["Unholy Runes"],
					desc = L["Unholy Runes"],
					hasAlpha = true,
					get = function(info) local v = Runes.db.colors.unholy; return v.r, v.g, v.b, v.a; end,
					set = function(info, re,gr,bl,al) Runes.db.colors.unholy = { r = re, g = gr, b = bl, a = al }; Runes:RuneWatch_Gui_Update("RuneWatch_Gui_Update", "null") end
				},
				frost = {
					order = 13,
					type = "color",
					name = L["Frost Runes"],
					desc = L["Frost Runes"],
					hasAlpha = true,
					get = function(info) local v = Runes.db.colors.frost; return v.r, v.g, v.b, v.a; end,
					set = function(info, re,gr,bl,al) Runes.db.colors.frost = { r = re, g = gr, b = bl, a = al }; Runes:RuneWatch_Gui_Update("RuneWatch_Gui_Update", "null") end
				},
				death = {
					order = 14,
					type = "color",
					name = L["Death Runes"],
					desc = L["Death Runes"],
					hasAlpha = true,
					get = function(info) local v = Runes.db.colors.death; return v.r, v.g, v.b, v.a; end,
					set = function(info, re,gr,bl,al) Runes.db.colors.death = { r = re, g = gr, b = bl, a = al }; Runes:RuneWatch_Gui_Update("RuneWatch_Gui_Update", "null") end
				},
			},
		}
	}
}

Runes.FontName = "Fonts\\FRIZQT__.TTF";
Runes.Font = CreateFont("RuneWatch_RuneFont");

function Runes:OnInitialize()
	if (not self:IsDeathKnight()) then
		return;
	end

	local defaults = {
		profile = {
			skin = "Blizzard",
			order = "buf",
			timer = {
				numerical = true,
				fontSize = 16,
			},
			colors = {
				blood = { r = 1, g = 0.4353, b = 0.4667, a = 1 },
				frost = { r = 0.5098, g = 0.5765, b = 1, a = 1 },
				unholy = { r = 0.5373, g = 1, b = 0.6431, a = 1 },
				death = { r = 0.8941, g = 0.4039, b = 1, a = 1 },
			}
		}
	}
	
	self:Register(Runes, RuneConfig.name, RuneConfig, defaults);
	self:InitRunes();
	
	-- Init the Font
	self:UpdateFont();
	
	self:RegisterMessage("RuneWatch_Gui_Update");
	self:RegisterMessage("RuneWatch_Gui_RuneMount");
	
	self:RegisterMessage("RuneWatch_Rune_Type");
	self:RegisterMessage("RuneWatch_Rune_State");
end

function Runes:OnEnable()
	self:Print("Enabled...");
end

function Runes:OnDisable()
	self:Print("Disabled...");
end

function Runes:InitRunes()
	self.Runes = {}
	
	for i = 1,6 do
		self.Runes[i] = self:InitRune(i);
	end
end

function Runes:InitRune(index)
	local frame = CreateFrame("frame", "RW_Rune"..index, UIParent);
	frame:SetWidth(32)
	frame:SetHeight(32)
	frame:SetPoint("CENTER", UIParent, "CENTER");
	frame:SetFrameStrata("HIGH");
	frame:SetFrameLevel(0);
	frame:Show();
	
	frame.ID = index;
	
	local texture = frame:CreateTexture("$parent_Background", "ARTWORK");
	frame.Image = texture
	texture:Show();
	
	texture:SetAllPoints(frame);
	texture:SetVertexColor(1,1,1,1);
	
	local cooldown = CreateFrame("cooldown", "$parent_Cooldown", frame);
	cooldown:SetPoint("CENTER", texture, "CENTER");
	cooldown:SetWidth(22);
	cooldown:SetHeight(22);
	cooldown:SetFrameStrata("HIGH");
	cooldown:SetFrameLevel(1);
	
	-- OmniCC Fix
	cooldown.noCooldownCount = true;
	
	local border = CreateFrame("frame", "$parent_Border", frame);
	border:SetAllPoints(frame);
	border:SetFrameStrata("HIGH");
	border:SetFrameLevel(2);	
	border:Show();
	frame.Border = border;
	
	local bart = border:CreateTexture("$parent_Art", "OVERLAY");
	bart:SetTexture(BorderTextures.Blizzard);
	bart:SetAllPoints(border);
	bart:Show();
	
	border.Art = bart;
	
	frame.Cooldown = cooldown;
	
	local t = border:CreateFontString("$parent_Timer", "OVERLAY", "SystemFont_Shadow_Huge1");
	frame.Timer = t;
	t:SetFontObject("RuneWatch_RuneFont");
	t:SetTextColor(1,1,0,1);
	
	-- Set Data
	t:SetText("0");
	t:SetAllPoints(frame);
	t:SetJustifyH("CENTER");
	t:SetJustifyV("MIDDLE");
	t:Hide();
	
	frame.Ready = true;
	
	frame:Show();
	
	return frame;
end

function Runes:UpdateFont()
	Runes.Font:SetFont(Runes.FontName, Runes.db.timer.fontSize);
end

Runes.RuneTypes = {
	[1] = "blood",
	[2] = "unholy",
	[3] = "frost",
	[4] = "death"
}

function Runes:RuneWatch_Rune_Type(self, ...)
	local rune, rtype = ...;
	
	local runeTheme = RuneTextures[Runes.db.skin] or RuneTextures.Blizzard;
	local runeBord = BorderTextures[Runes.db.skin] or BorderTextures.Blizzard;
	
	local runeArt = runeTheme[rtype];
	
	Runes.Runes[rune].Image:SetTexture(runeArt);
	Runes.Runes[rune].Border.Art:SetTexture(runeBord);
	
	local runeTimerCode = (Runes.RuneTypes[rtype] or "blood");
	local runeTimerColor = Runes.db.colors[runeTimerCode];
	
	Runes.Runes[rune].Timer:SetTextColor(runeTimerColor.r,
										 runeTimerColor.g,
										 runeTimerColor.b,
										 runeTimerColor.a);
end

function Runes:RuneWatch_Rune_State(self, ...)
	local rune, usable = ...;
	
	--[[if (usable) then
		Runes.Runes[rune].Ready = true;
	else
		Runes.Runes[rune].Ready = false;
	end]]--
	
	if not usable then
		Runes.Runes[rune]:SetScript("OnUpdate", function(self, elapsed) Runes:Rune_OnUpdate(self, elapsed) end);
		
		if (Runes.db.timer.numerical) then
			Runes.Runes[rune].Timer:Show();
		else
			Runes.Runes[rune].Timer:Hide();
		end
		
		Runes.Runes[rune].Image:SetVertexColor(0.8,0.8,0.8,0.9);
	end
end

function Runes:RuneWatch_Gui_Update(self, ...)
	for i = 1, 6 do		
		RuneWatch:SendMessage("RuneWatch_Rune_Type", i, GetRuneType(i));
		
		local rune = Runes.Runes[i];
		
		--[[if (Runes.db.timer.numerical) then
			rune.Timer:Show();
		else
			rune.Timer:Hide();
		end]]--
	end
end

Runes.Map = {
	["buf"] = {	[1] = 1, [2] = 2, [3] = 3, [4] = 4, [5] = 5, [6] = 6 },
	["bfu"] = {	[1] = 1, [2] = 2, [3] = 5, [4] = 6, [5] = 3, [6] = 4 },
	["ubf"] = {	[1] = 3, [2] = 4, [3] = 1, [4] = 2, [5] = 5, [6] = 6 },
	["ufb"] = {	[1] = 3, [2] = 4, [3] = 5, [4] = 6, [5] = 1, [6] = 2 },
	["fbu"] = {	[1] = 5, [2] = 6, [3] = 1, [4] = 2, [5] = 3, [6] = 4 },
	["fub"] = {	[1] = 5, [2] = 6, [3] = 3, [4] = 4, [5] = 1, [6] = 2 },
}
function Runes:RuneWatch_Gui_RuneMount(self, ...)
	Runes:Print("Caught Mount Event");

	local data = ...;
	
	for i = 1,6 do
		local ind = Runes.Map[Runes.db.order][i];
		
		Runes.Runes[ind]:SetParent(data.parent);
		Runes.Runes[ind]:ClearAllPoints();
		Runes.Runes[ind]:Show();
		Runes.Runes[ind]:SetPoint(unpack(data.runes[i]));
	end
end

function Runes:Rune_OnUpdate (self, elapsed)
	local start, duration, runeReady = GetRuneCooldown(self.ID);
	
	local displayCooldown = (runeReady and 0) or 1;

	if (displayCooldown) then
		local offset = GetTime() - start;
		local timer = ceil(duration - offset);
		
		-- Fix for rune-timeout when dead
		if (timer <= 0) then
			self.Timer:Hide();
			self.Image:SetVertexColor(1,1,1,1);
			self:SetScript("OnUpdate", nil);
		else
			self.Timer:SetText(timer);
			self.Cooldown:SetCooldown(start, duration);
		end
	end
	
	if ( runeReady ) then
		-- Stop Updating! Unregister the script
		self.Timer:Hide();
		self.Image:SetVertexColor(1,1,1,1);
		self:SetScript("OnUpdate", nil);
	end
end
